# -*- coding: utf-8 -*-
"""
Created on 2020/4/22 12:08 AM
---------
@summary:
---------
@author: Boris
@email: boris_liu@foxmail.com
"""

__all__ = ["AirSpider", "Spider", "BatchSpider"]

from feapder.core.spiders.air_spider import AirSpider
from feapder.core.spiders.spider import Spider
from feapder.core.spiders.batch_spider import BatchSpider
